;******************************************************************************
;                                                                             *
;    Filename:      YD36SAP_DU2                                               *
;    Date:          1.5.2010                                                  *
;    File Version:  1.0                                                       *
;                                                                             *
;    Author:        Martin Kvetko                                             *
;    Company:       FEL CVUT                                                  *
;    Descriptin:    Citac od 0-255 s periodou 800ms							  *
;					Funkce RA5 = RESET			                              *                        
;					Na PORD zobrazit vysledek funkce:						  *
;					y=K2*x^2+K1*x+K0										  *
;	                kde x = hodnota citace									  *
;					K2=11;K1=6;K0=11										  *
;					Citac bude mozne zastavovata spoustet stiskem RB0		  *
;											                                  * 
;******************************************************************************
;                                                                             *
;    Files Required: P18F87J11.INC                                            *
;                                                                             *
;******************************************************************************

	LIST P=18F87J11		;directive to define processor
	#include <P18F87J11.INC>	;processor specific variable definitions

;******************************************************************************
;Configuration bits
;Microchip has changed the format for defining the configuration bits, please 
;see the .inc file for futher details on notation.  Below are a few examples.


;   Oscillator Selection:
    CONFIG	FOSC = HS            ;HS oscilator
	CONFIG	WDTEN = OFF;         ;nepouzivame Watchdog
	CONFIG	XINST = OFF;		 ;nepouzivame rozsireny instrukcni soubor

;Preddefinovane konstanty
#define K2			.11
#define K1			.6
#define K0			.11
#define TMR0_H 		0xCF
#define TMR0_L 		0x2B

#define PRESSED		1
#define NOTPRESSED	0

#define STATE_STOP	0
#define STATE_RUN	1


;******************************************************************************
;Variable definitions
; These variables are only needed if low priority interrupts are used. 
; More variables may be needed to store other special function registers used
; in the interrupt routines.



	udata_acs 0x000
RESULT 		res 3	;Vysledek funkce
WREG_TEMP 	res 1	;variable used for context saving 
STATUS_TEMP res 1	;variable used for context saving
BSR_TEMP 	res 1	;variable used for context saving

RB0_STATUS 	res 1	;tlacitko RB0 status
RA5_STATUS  res 1	;tlacitko RA5 status
STATE 		res 1	;promenna udrzujici informaci o stavu
COUNTER 	res 1	;Citac

;******************************************************************************
;Reset vector
; This code will start executing when a reset occurs.

		ORG	0x0000			;startovaci adresa po resetu

		goto	Main		;skok na hlavni cast programu

;******************************************************************************
;High priority interrupt vector
; This code will start executing when a high priority interrupt occurs or
; when any interrupt occurs if interrupt priorities are not enabled.

		ORG	0x0008			;vektor preruseni pro vysokou prioritu

		goto	HighInt		;skok na ISR pri preruseni s vysokou prioritou

;******************************************************************************
;Low priority interrupt vector and routine
; This code will start executing when a low priority interrupt occurs.
; This code can be removed if low priority interrupts are not used.

		ORG	0x0018			;vektor preruseni pro nizkou prioritu

		movff	STATUS,STATUS_TEMP	;save STATUS register
		movff	WREG,WREG_TEMP		;save working register
		movff	BSR,BSR_TEMP		;save BSR register
		
		movlw	NOTPRESSED
		btfss	PORTB, 0, 0			;Je RB0 stisknuto?
		movlw	PRESSED
		movwf	RB0_STATUS			;Nastav aktualni informaci o zmacknuti tlacitka


		btfss 	PORTA,5				;Kdyz RA5 stisknuto vymaz citac
		clrf	COUNTER

		bcf		PIR1, TMR1IF, 0		;vynuluj priznak prave osetreneho preruseni

		movff	BSR_TEMP,BSR		;restore BSR register
		movff	WREG_TEMP,WREG		;restore working register
		movff	STATUS_TEMP,STATUS	;restore STATUS register
		retfie						;navrat z ISR

;******************************************************************************
;High priority interrupt routine
; The high priority interrupt code is placed here to avoid conflicting with
; the low priority interrupt vector.

HighInt:
		movlw	TMR0_H	;nastaveni pocatecni hodnoty Timeru 0
		movwf	TMR0H
		movlw	TMR0_L
		movwf	TMR0L

		movlw	STATE_STOP			;pokud v STATE_RUN zvys hodnotu TIME
		cpfseq	STATE
		bra		HighIntTimeRunning
		bra		HighIntEnd

HighIntTimeRunning:		
		incf	COUNTER
		call    Count
		
HighIntEnd:		
		bcf		INTCON, TMR0IF, 0	;vynuluj priznak prave osetreneho preruseni
		movff	COUNTER, PORTD		;Zobraz aktualni stav na LEDkach
		retfie	FAST				;navrat z ISR s obnovenim kontextu

Count:
		clrf 	PROD				;nulovani vsech promennych pro dalsi vypocet
		clrf 	0x000
		clrf 	0x001
		clrf 	0x002
		
		movff 	COUNTER,WREG    ;COUNTER to WREG
		mulwf 	COUNTER		   	;Product COUNTER*COUNTER
		movff 	PRODL, 0x000	;PRODLOW to First BYTE
		movff 	PRODH, 0x001	;PRODHI to second BYTE
		movlw 	K2				;K2 ro WREG
		mulwf	0x001			;K2*second byte
		movff 	PRODH,0x002		;hi byte of product to third byte
		movff 	PRODL,0x001		;lo byte of product to third byte
		movlw	K2				
		mulwf	0x000			;K2*first byte
		movff   PRODL,0x000		;lo byte of product to first byte		
		movff   PRODH,WREG		
		addwfc	0x001			;hi byte of product to add with second byte
		btfsc 	STATUS,C		;second byte overflows? go to overflow rutine
		call 	Over2
		
		movlw 	K1				
		mulwf	COUNTER			;K1*COUNTER
		movff 	PRODL,WREG
		addwfc 	0x000			;L-BYTE + RESULT
		btfsc 	STATUS,C		;Overflow?
		call 	Over1
		movff 	PRODH,WREG		
		addwfc 	0x001
		btfsc 	STATUS,C		;Overflow?
		call 	Over2
		
		movlw 	K0				;K0
		addwfc 	0x000			;RESULT+K0
		btfsc 	STATUS,C		;Overflow?
		call 	Over1
		return

Over1:
		incf	0x001
		btfsc 	STATUS,OV
		call 	Over2
		return
Over2:
		incf 	0x002
		return
		

;******************************************************************************
;Start of main program
; The main program code is placed here.

Main:
		call 	Init

Loop:
		call	StateMachine
		goto	Loop				;nekonecna smycka


Init:	clrf	TRISD, 0		;Konfiguruace PORTD jako vystup
		clrf	PORTD, 0		;Inicializace hodnot na PORTD
		clrf	RB0_STATUS		;Inicializace uzivatelskych promennych
		clrf	STATE
		clrf	COUNTER
		clrf 	RESULT
		
		movlw	TMR0_H	;nastaveni pocatecni hodnoty Timeru0
		movwf	TMR0H
		movlw	TMR0_L
		movwf	TMR0L
		
		; PORTA RA5
    	bsf     TRISA,RA5
		; nastaveni RA5 jako cislicovy vstup
    	;bsf WDTCON,ADSHR,0	;otevreni registru
    	;bsf	ANCON0,PCFG4,0	; RA5 je cislicovy
    	;bcf	WDTCON,ADSHR,0	; uzavreni registru
		
		;movlw	b'00000001'		;Konfigurace PORTB jako vystup az na bit0
		;movwf	TRISB
		

 		bsf     TRISA,RA5,ACCESS
		; nastaveni RA5 jako cislicovy vstup
    	bsf WDTCON,ADSHR,0	;otevreni registru
    	bsf	ANCON0,PCFG4,0	; RA5 je cislicovy
    	bcf	WDTCON,ADSHR,0	; uzavreni registru
    	; PORTB RB0
    	bsf     TRISB,RB0,ACCESS

	
	
		movlw	b'00000111'		;Timer0 ma 16b, z internich hodin, s 1:256 prescalerem
		movwf	T0CON

		movlw	b'10000000'		;Timer1 ma 16b, z internich hodin, s 1:8 prescalerem
		movwf	T1CON

		movlw	b'11100000'		;Preruseni od preteceni Timer0 povoleno
		movwf	INTCON

		movlw	b'00000100'		;Preruseni od Timer0 ma vysokou prioritu
		movwf	INTCON2

		movlw	b'10000000'		;Povoleni priorit u preruseni
		movwf	RCON

		bsf		PIE1, TMR1IE, 0	;Povol preruseni od Timer1
		bcf 	IPR1, TMR1IP, 0	;Preruseni od Timer1 ma nizkou prioritu
		
		bsf		T0CON, TMR0ON, 0 ;Spust Timer0
		bsf		T1CON, TMR1ON, 0 ;Spust Timer1
	
		return

StateMachine:
		movlw	NOTPRESSED
Press1:	cpfseq	RB0_STATUS		;Bylo RB0 uvolneno pri preruseni?
		bra		Press1
		movlw	PRESSED
Press2:	cpfseq	RB0_STATUS		;Bylo RB0 zmacknuto pri preruseni?
		bra		Press2

		btg		STATE, 0, 0	;Zmen stav RUN/STOP
	
		return
	
		END